#
# context.rb - the minimum set to start Wakaba.
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: context.rb,v 1.1 2002/09/10 11:07:27 katsu Exp $
#

begin
  require 'stringio'
rescue LoadError
  require 'wakaba/strio'
end

require 'wakaba/error'


module Wakaba

  class HTTPHeader

    include Enumerable


    def initialize
      @hash = {}
      @hash.default = [].freeze
    end


    private

    RFC1123Day = %w(Sun Mon Tue Wed Thu Fri Sat)
    RFC1123Month = %w(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec)

    def rfc1123_date(time)
      t = time.dup.gmtime
      sprintf("%s, %02d %s %04d %02d:%02d:%02d GMT",
              RFC1123Day[t.wday], t.day, RFC1123Month[t.month - 1], t.year,
              t.hour, t.min, t.sec)
    end

    def to_out(v)
      if Time === v then
        rfc1123_date v
      else
        v.to_s
      end
    end


    public

    def get(key)
      @hash[key].first
    end

    alias []  get

    def getall(key)
      @hash[key].dup
    end

    def out(key)
      @hash[key].collect{ |i| to_out i }
    end


    def each
      @hash.each { |key,val|
        val.each { |i| yield key, i }
      }
      self
    end

    def each_out
      @hash.each { |key,val|
        # these are illigal for HTTP headers. ignore them.
        # /[^()<>@,;:\\"\/\[\]?={} \t\000-\037\177-\255]+/ =~ token (RFC2616)
        next if /[\000-\040"()\/\072-\100\133-\135{}\177-\377]/n =~ key or
                key.empty?
        key = key.gsub(/\b(.)/) { $1.upcase }
        val.each { |i| yield key, to_out(i) }
      }
      self
    end

    def keys
      @hash.keys
    end


    def to_hash
      dst = {}
      @hash.each { |key,val| dst[key] = val.first }
      dst
    end


    def set(key, val)
      if val then
        @hash[key] = [ val ]
      else
        @hash.delete key
      end
      val
    end

    alias []=  set

    def add(key, val)
      if val then
        if @hash.key? key then
          @hash[key].push val
        else
          @hash[key] = [ val ]
        end
      end
      val
    end

  end




  class Context

    def _dump(limit)
      raise TypeError, "can't dump Wakaba::Context"
    end


    def initialize(controller, interface, session_map, session_name, action)
      @controller = controller
      @interface = interface
      @session_map = session_map
      @session_name = session_name
      @action = action
      @header_out = HTTPHeader.new
      @body_out = StringIO.new
    end

    attr_reader :session_name, :action, :header_out

    alias response_header header_out


    def body_out
      @body_out.string
    end

    alias response_body  body_out


    def write(str)
      @body_out.write str
    end

    def <<(str)
      @body_out << str
      self
    end

    def print(*args)
      @body_out.print(*args)
    end

    def printf(*args)
      @body_out.printf(*args)
    end

    def puts(*args)
      @body_out.puts(*args)
    end

    def putc(c)
      @body_out.putc c
    end


    CGIEnv = %w(
      SERVER_SOFTWARE   SERVER_NAME      GATEWAY_INTERFACE
      SERVER_PROTOCOL   REQUEST_METHOD   PATH_INFO
      PATH_TRANSLATED   SCRIPT_NAME      QUERY_STRING      REMOTE_HOST
      REMOTE_ADDR       AUTH_TYPE        REMOTE_USER       REMOTE_IDENT
      CONTENT_TYPE      SERVER_PORT      CONTENT_LENGTH
    )
    CGIEnv.freeze
    CGIEnv.each { |i| i.freeze }

    CGIEnv[0...-2].each { |i|
      d = i.downcase
      module_eval "def #{d}; @interface.cgienv '#{i}'; end"
    }
    def server_port    ; @interface.cgienv('SERVER_PORT').to_i    ; end
    def content_length ; @interface.cgienv('CONTENT_LENGTH').to_i ; end

    def header_in(key)
      @interface.header_in key
    end
    alias request_header  header_in

    def param
      @interface.param
    end


    def href(session, action = nil)
      session_name = @session_map.mount(session)
      action = session.__class__.method_to_action(action) if action
      @interface.href(session_name, action)
    end

    def base_uri
      @interface.base_uri
    end

    alias script_base_uri  base_uri


    def lookup(session_name)
      @session_map.lookup session_name
    end

    def mount(session, session_name = nil)
      @session_map.mount session, session_name
    end

    def umount(session)
      @session_map.umount session
    end

    def save_session
      @controller.save_session
    end

    def finale
      @controller.finale
    end

  end

end
