#
# wakaba/session.rb
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: session.rb,v 1.1 2002/09/10 11:07:27 katsu Exp $
#

require 'thread'
require 'wakaba/error'


module Wakaba

  class Session

    # only for internal use. User must not override this method.
    alias __class__ class


    def deny(msg = nil)
      raise(AccessDenied, msg || 'the access is not allowed')
    end
    private :deny


    def self.action_to_method(action)
      if action.empty? then
        :welcome
      else
        ('on_' + action).intern
      end
    end


    def self.method_to_action(action)
      if Symbol === action then
        if action == :welcome then
          ''
        else
          unless /\Aon_/ =~ action.id2name then
            raise ActionNameError, "invalid symbol #{action} as an action"
          end
          $'
        end
      elsif String === action then
        action
      else
        raise TypeError, "invalid type #{action.class.name} as an action"
      end
    end


    def dispatch(context)
      method = __class__.action_to_method(context.action)
      unless respond_to? method then
        action_missing context
      else
        setup context
        begin
          send method, context
        ensure
          teardown context
        end
      end
    end


    private

    # These methods are completely freed for users, so they must do nothing
    # or only one specified thing by default. Don't override these methods
    # for internal use.

    def action_missing(context)
      raise ActionNotFound, "undefined action `#{context.action}'"
    end

    def setup(context)
    end

    def teardown(context)
    end

  end




  class SessionMap

    def initialize(hash = nil)
      @hash = hash || {}
      @reverse = nil
      @mutex = Mutex.new
    end

    attr_reader :hash


    private

    def reverse_fetch(session, default = nil)
      unless @reverse then
        @reverse = {}
        @hash.each { |k,v| @reverse[v.__id__] = k }
      end
      @reverse.fetch session.__id__, default
    end

    def valid_session_name?(session_name)
      /\A(?:\/[-.\w]+)+\z/n =~ session_name
    end

    def make_session_name(session)
      prefix = "/#{session.__class__.name.sub(/\A.*::/n, '')}"
      unless @hash.key? prefix then
        prefix
      else
        n = 0
        begin
          n += 1
          s = prefix + '-%04d' % n
        end while @hash.key? s
        s
      end
    end


    public

    def lookup(session_name)
      session = @hash[session_name]
      raise SessionNameError,"no such session `#{session_name}'" unless session
      session
    end


    def mount(session, session_name = nil)
      raise TypeError, 'must be a Wakaba::Session' unless Session === session
      @mutex.synchronize {
        key = reverse_fetch(session)
        if key then
          if session_name and key != session_name then
            raise SessionTypeError, 'already mounted by another session name'
          end
          key
        else
          session_name ||= make_session_name(session)
          unless valid_session_name? session_name then
            raise SessionNameError, "invalid session ID #{session_name.dump}"
          end
          s = @hash[session_name]
          if s and not s.equal? session then
            raise(SessionNameError,
                  "session ID `#{session_name}' is used for another session")
          end
          @hash[session_name] = session
          @reverse[session.__id__] = session_name
          session_name
        end
      }
    end

    def umount(session)
      @mutex.synchronize {
        key = reverse_fetch(session)
        raise SessionTypeError, "unmounted session" unless key
        @hash.delete key
        @reverse.delete session.__id__
        key
      }
    end

  end

end
