#
# wakaba/store.rb
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: store.rb,v 1.2 2002/09/10 13:16:12 katsu Exp $
#

require 'wakaba/error'
require 'wakaba/version'


module Wakaba

  class Record

    def index=(index)
      @index = String.new(index)
      index
    end

    def server_key=(key)
      @server_key = String.new(key)
      key
    end

    def version=(version)
      @version = String.new(version)
      version
    end

    def ctime=(time)
      raise TypeError, "ctime must be a Time" unless Time === time
      @ctime = time
    end

    def alive=(f)
      @alive = !!f
      f
    end

    attr_writer :object
    attr_reader :index, :server_key, :version, :ctime, :object, :alive
    alias alive? alive


    def initialize(index, object = nil, server_key = nil,
                   version = nil, ctime = nil)
      self.index = index
      self.server_key = server_key || ''
      self.version = version || ''
      self.ctime = ctime || Time.at(0)
      @object = object
      @alive = true
    end

    class << self
      alias [] new
    end

    def identity
      ''   # no identity
    end

  end




  class MarshallingRecord < Record

    def initialize(index, marshalled_object = nil, server_key = nil,
                   version = nil, ctime = nil)
      @marshalled_object = marshalled_object
      object = marshalled_object && Marshal.load(marshalled_object)
      super index, object, server_key, version, ctime
    end

    def object
      @marshalled_object = nil
      super
    end

    def object=(arg)
      @marshalled_object = nil
      super
    end

    def marshal_object
      @marshalled_object or Marshal.dump(@object)
    end

    def identity
      marshal_object
    end

  end




  class Store

    def initialize(version = nil, max_age = nil)
      @version = version && version.to_str
      @max_age = max_age && max_age.to_int
      @now = nil
    end


    def now=(t)
      raise TypeError, "now must be nil or a Time" if t and not Time === t
      @now = t
    end

    def now
      @now or Time.at(Time.now.to_i)
    end

    def version
      wakaba_version = Version.sub(/\.[^.]*\z/, '')
      "#{@version || ''}.#{wakaba_version}"
    end


    private

    def check_record(record)
      if @version and version != record.version then
        raise IncompatibleSession, "incompatible session"
      end
      if @max_age and now - @max_age > record.ctime then
        raise ExpiredSession, "this session has been expired"
      end
      unless record.alive? then
        raise ExpiredSession, "this session has been dead"
      end
    end


    def lock_failed
      raise LockFailed, "failed locking a record"
    end


    def do_open(index)
      # open a record. With lock or without lock is not mensioned.
      do_open_ex index
    end

    def do_open_ex(index)
      # open a record exclusively.
      raise LockUnsupported, "record locking is not supported"
    end

    def do_create
      # create a new record.
      raise AbstractMethod
    end


    public

    def open(index, lock)
      ret, err = nil
      unless index then
        do_create { |record|
          index = record.index.dup
          ret = yield(record)
          record.index = index
          record.version = version
          record.ctime = now
        }
      else
        if lock then
          open = :do_open_ex
        else
          open = :do_open
        end
        __send__(open, index) { |record|
          index = record.index.dup
          begin
            check_record record
          rescue AccessError => err
            record.alive = false
          end
          ret = yield(record) unless err
          record.index = index
          record.version = version
          record.ctime = now
        }
        raise err, err.message, err.backtrace if err
      end
      ret
    end

  end




  class EchoStore < Store

    private

    def encode(s)
      s = [s].pack('m*')
      s.gsub!(/\s+/, '')
      s.tr!('=', '_')
      s
    end

    def decode(s)
      s.tr('_', '=').unpack('m*')[0]
    end

    def escape(s)
      s = s.gsub(/\./, '..')
      s.gsub!(/-/, '._')
      s
    end

    def unescape(s)
      s = s.gsub(/\._/, '-')
      s.gsub!(/\.\./, '.')
      s
    end


    def save(record)
      ctime = record.ctime.to_i
      marshalled = encode(record.marshal_object)
      server_key = escape(record.server_key)
      ver = record.version
      record.index = "#{ctime}-#{marshalled}-#{server_key}-#{ver}"
    end


    public

    def do_create
      record = MarshallingRecord.new('')
      yield record
      save record
    end


    def do_open(index)
      ctime, marshalled, server_key, ver = index.split('-', 4)
      unless ctime and /\A\d+\z/ =~ ctime and ver then
        raise InvalidSessionIndex, 'invalid session index'
      end
      ctime = Time.at(ctime.to_i)
      marshalled = decode(marshalled)
      server_key = unescape(server_key)
      record = MarshallingRecord.new(index, marshalled, server_key, ver, ctime)
      yield record
      save record
    end

  end

end
