#
# testcontext.rb - test cases for Wakaba
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: testcontext.rb,v 1.1 2002/09/10 11:07:27 katsu Exp $
#

require 'runit/testcase'
require 'asserttracecall'
require 'wakaba/context'
require 'wakaba/session'


class TestHTTPHeader < RUNIT::TestCase

  def setup
    @h = Wakaba::HTTPHeader.new
    @h['hoge'] = 'fuga'
    @h['foo'] = 12345678
    @h.add 'foo', 23456789
    @t = Time.local(2000, 7, 8, 9, 10, 11)
    @h['fuga'] = @t
    @h['Hoge-Fuga'] = 'fugahoge'
    @h['Hoge Fuga'] = 'fugahogefuga'
  end


  def test_enumerable
    assert_equal true, Wakaba::HTTPHeader.ancestors.include?(Enumerable)
  end


  def test_get
    assert_equal 'fuga', @h['hoge']
    assert_equal 12345678, @h['foo']
    assert_equal @t, @h['fuga']
    assert_nil @h['bar']
  end

  def test_get_casesensitive
    assert_nil @h['Hoge']
    assert_nil @h['HOGE']
    assert_nil @h['hoge-fuga']
  end


  def test_getall
    assert_equal ['fuga'], @h.getall('hoge')
    assert_equal [12345678,23456789], @h.getall('foo')
    assert_equal [@t], @h.getall('fuga')
  end

  def test_getall_casesensitive
    assert_equal [], @h.getall('Hoge')
    assert_equal [], @h.getall('HOGE')
    assert_equal [], @h.getall('hoge-fuga')
  end

  def test_getall_destroy
    a = @h.getall('foo')
    a.push 'hoge'
    assert_equal [12345678,23456789,'hoge'], a
    assert_equal [12345678,23456789], @h.getall('foo')
  end


  def test_out
    assert_equal ['fuga'], @h.out('hoge')
    assert_equal ['12345678','23456789'], @h.out('foo')
    assert_equal ['Sat, 08 Jul 2000 00:10:11 GMT'], @h.out('fuga')
  end

  def test_out_casesensitive
    assert_equal [], @h.out('Hoge')
    assert_equal [], @h.out('HOGE')
    assert_equal [], @h.out('hoge-fuga')
  end

  def test_out_gmtime
    @h['fuga'] = @t.dup.gmtime
    assert_equal ['Sat, 08 Jul 2000 00:10:11 GMT'], @h.out('fuga')
  end


  def test_each
    a = []
    @h.each { |k,v| a.push [k,v] }
    a.sort!
    assert_equal([ ['Hoge Fuga','fugahogefuga'],
                   ['Hoge-Fuga','fugahoge'],
                   ['foo',12345678],
                   ['foo',23456789],
                   ['fuga',@t],
                   ['hoge','fuga'],
                 ], a)
  end


  def test_each_out
    a = []
    @h.each_out { |k,v| a.push [k,v] }
    a.sort!
    assert_equal([ ['Foo','12345678'],
                   ['Foo','23456789'],
                   ['Fuga','Sat, 08 Jul 2000 00:10:11 GMT'],
                   ['Hoge','fuga'],
                   ['Hoge-Fuga','fugahoge'],
                 ], a)
  end


  def test_to_hash
    assert_equal({ 'Hoge Fuga' => 'fugahogefuga',
                   'Hoge-Fuga' => 'fugahoge',
                   'foo' => 12345678,
                   'fuga' => @t,
                   'hoge' => 'fuga',
                 }, @h.to_hash)
  end


  def test_keys
    assert_equal ['Hoge Fuga','Hoge-Fuga','foo','fuga','hoge'], @h.keys.sort
  end


  def test_set_new
    s = 'meso'
    assert_same s, (@h['bar'] = s)
    assert_same s, @h['bar']
  end

  def test_set_update
    s = 'meso'
    @h['hoge'] = s
    assert_same s, @h['hoge']
    @h['foo'] = s
    assert_same s, @h['foo']
    assert_equal [s], @h.out('foo')
  end

  def test_set_nil
    @h['hoge'] = nil
    assert_nil @h['hoge']
    assert_equal [], @h.out('hoge')
    @h['foo'] = nil
    assert_nil @h['foo']
    assert_equal [], @h.out('foo')
  end

  def test_set_casesensitive
    @h['Hoge'] = 'hoge'
    assert_equal ['fuga'], @h.getall('hoge')
    assert_equal [], @h.getall('HOGE')
    assert_equal ['hoge'], @h.getall('Hoge')
  end


  def test_add
    s = 'meso'
    assert_same s, @h.add('hoge', s)
    assert_equal 'fuga', @h['hoge']
    assert_equal ['fuga','meso'], @h.out('hoge')
    @h.add 'foo', s
    assert_equal 12345678, @h['foo']
    assert_equal ['12345678','23456789','meso'], @h.out('foo')
  end

  def test_add_as_set
    @h.add 'bar', 'meso'
    assert_equal 'meso', @h['bar']
    assert_equal ['meso'], @h.out('bar')
  end

  def test_add_nil
    assert_same nil, @h.add('hoge', nil)
    assert_equal 'fuga', @h['hoge']
    assert_equal ['fuga'], @h.out('hoge')
  end

  def test_add_casesensitive
    @h.add 'Hoge', 'hoge'
    assert_equal ['fuga'], @h.getall('hoge')
    assert_equal [], @h.getall('HOGE')
    assert_equal ['hoge'], @h.getall('Hoge')
  end

end




class TestContext < RUNIT::TestCase

  class DummyController
    def save_session
      'hogesave'
    end
    def finale
      'hogefinale'
    end
  end

  class DummyInterface
    def cgienv(key)
      if key == 'SERVER_PORT' or key == 'CONTENT_LENGTH' then
        '12345678fugacgienv'
      else
        'fugacgienv'
      end
    end
    def header_in(key)
      'fugain'
    end
    def param
      'fugaparam'
    end
    def href(session_name, action)
      'fugahref'
    end
    def base_uri
      'fugauri'
    end
  end


  def setup
    @s1 = Wakaba::Session.new
    @s2 = Wakaba::Session.new
    @sessionmap = Wakaba::SessionMap.new({'/hoge' => @s1 })
    @controller = DummyController.new
    @interface = DummyInterface.new
    @c = Wakaba::Context.new(@controller, @interface, @sessionmap,
                             'hogehoge', 'fugafuga')
  end


  def test_undumpable
    assert_exception(TypeError) { Marshal.dump @c }
  end


  def test_session_name
    assert_equal 'hogehoge', @c.session_name
    @c.session_name << 'fuga'
    assert_equal 'hogehogefuga', @c.session_name
  end


  def test_action
    assert_equal 'fugafuga', @c.action
    @c.action << 'hoge'
    assert_equal 'fugafugahoge', @c.action
  end


  def test_header_out
    assert_kind_of Wakaba::HTTPHeader, @c.header_out
  end


  def test_body_out
    assert_equal '', @c.body_out
    @c.print 'hoge'
    assert_equal 'hoge', @c.body_out
  end


  def test_write
    assert_equal 4, @c.write('hoge')
    assert_equal 'hoge', @c.body_out
    assert_equal 4, @c.write('fuga')
    assert_equal 'hogefuga', @c.body_out
  end


  def test_addstr
    assert_same @c, (@c << 'hoge')
    assert_equal 'hoge', @c.body_out
    assert_same @c, (@c << 'fuga')
    assert_equal 'hogefuga', @c.body_out
  end


  def test_print
    assert_nil @c.print('hoge')
    assert_equal 'hoge', @c.body_out
    assert_nil @c.print('fuga', 'hoge')
    assert_equal 'hogefugahoge', @c.body_out
  end


  def test_printf
    assert_nil @c.printf('hoge')
    assert_equal 'hoge', @c.body_out
    assert_nil @c.printf('%sfuga', 'hoge')
    assert_equal 'hogehogefuga', @c.body_out
  end


  def test_puts
    assert_nil @c.puts('hoge')
    assert_equal "hoge\n", @c.body_out
    assert_nil @c.puts('fuga', 'hoge')
    assert_equal "hoge\nfuga\nhoge\n", @c.body_out
  end


  def test_putc
    assert_equal ?h, @c.putc(?h)
    assert_equal 'h', @c.body_out
    assert_equal 'o', @c.putc('o')
    assert_equal 'ho', @c.body_out
  end


  CGIEnvStr = [
    'AUTH_TYPE', 'CONTENT_TYPE', 'GATEWAY_INTERFACE',
    'PATH_INFO', 'PATH_TRANSLATED', 'QUERY_STRING', 'REMOTE_ADDR',
    'REMOTE_HOST', 'REMOTE_IDENT', 'REMOTE_USER', 'REQUEST_METHOD',
    'SCRIPT_NAME', 'SERVER_NAME', 'SERVER_PROTOCOL',
    'SERVER_SOFTWARE'
  ]
  CGIEnvInt = [ 'CONTENT_LENGTH', 'SERVER_PORT' ]

  def test_cgienv
    assert_equal((CGIEnvStr + CGIEnvInt).sort, Wakaba::Context::CGIEnv.sort)
  end


  CGIEnvStr.each { |i|
    d = i.downcase
    module_eval <<-END, __FILE__, __LINE__ + 1
      def test_#{d}
        assert_trace_call([@interface, :cgienv, {'key'=>'#{i}'}]) {
          assert_equal 'fugacgienv', @c.#{d}
        }
      end
    END
  }
  CGIEnvInt.each { |i|
    d = i.downcase
    module_eval <<-END, __FILE__, __LINE__ + 1
      def test_#{d}
        assert_trace_call([@interface, :cgienv, {'key'=>'#{i}'}]) {
          assert_equal 12345678, @c.#{d}
        }
      end
    END
  }


  def test_header_in
    assert_trace_call([@interface, :header_in, {'key'=>'Hoge'}]) {
      assert_equal 'fugain', @c.header_in('Hoge')
    }
  end


  def test_param
    assert_trace_call([@interface, :param]) {
      assert_equal 'fugaparam', @c.param
    }
  end


  def test_href
    o = Object.new
    assert_trace_call([@sessionmap, :mount,
                        {'session'=>@s1, 'session_name'=>nil}],
                      [@interface, :href,
                        {'session_name'=>'/hoge', 'action'=>'foobar'}]) {
      assert_equal 'fugahref', @c.href(@s1, 'foobar')
    }
  end

  def test_href_unmounted
    assert_trace_call([@sessionmap, :mount,
                        {'session'=>@s2, 'session_name'=>nil}],
                      [@interface, :href,
                        {'session_name'=>'/Session', 'action'=>nil}]) {
      assert_equal 'fugahref', @c.href(@s2)
    }
  end

  def test_href_symbol
    assert_trace_call([@sessionmap, :mount,
                        {'session'=>@s2, 'session_name'=>nil}],
                      [@interface, :href,
                        {'session_name'=>'/Session', 'action'=>'foobar'}]) {
      assert_equal 'fugahref', @c.href(@s2, :on_foobar)
    }
  end


  def test_base_uri
    assert_trace_call([@interface, :base_uri]) {
      assert_equal 'fugauri', @c.base_uri
    }
  end


  def test_mount
    assert_trace_call([@sessionmap, :mount,
                        {'session'=>@s2, 'session_name'=>'/foobar'}]) {
      assert_equal '/foobar', @c.mount(@s2, '/foobar')
    }
  end

  def test_mount_omit
    assert_trace_call([@sessionmap, :mount,
                        {'session'=>@s2, 'session_name'=>nil}]) {
      assert_equal '/Session', @c.mount(@s2)
    }
  end


  def test_umount
    assert_trace_call([@sessionmap, :umount, {'session'=>@s1}]) {
      assert_equal '/hoge', @c.umount(@s1)
    }
  end


  def test_save_session
    assert_trace_call([@controller, :save_session]) {
      assert_equal 'hogesave', @c.save_session
    }
  end


  def test_finale
    assert_trace_call([@controller, :finale]) {
      assert_equal 'hogefinale', @c.finale
    }
  end

end




load "#{File.dirname($0)}/runtest.rb" if __FILE__ == $0
