#
# testerb.rb - test cases for Wakaba
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: testerb.rb,v 1.13 2002/07/29 13:02:55 katsu Exp $
#

require 'runit/testcase'
require 'wakaba/erb'
require 'wakaba/strio'
require 'erb/erbl'

def erb_version
  ERbLight.version.split(/[ \[]+/)[1]
end



class TestERbView < RUNIT::TestCase

  class DummyContext
    def initialize
      @out = Wakaba::StringIO.new
    end
    def write(s)
      @out.write s
    end
    def puts(*args)
      @out.puts(*args)
    end
    def print(*args)
      @out.print(*args)
    end
    def printf(*args)
      @out.printf(*args)
    end
    def href(a, b = nil)
      [ :href, a, b ]
    end
    def out
      @out.string
    end
  end


  TestFile = '/tmp/wakabaerbtest'
  TestFile2 = '/tmp/wakabaerbtest2'

  def create_test_file
    File.open(TestFile, 'w') { |f| f.write yield }
  end

  def create_test_file_2
    File.open(TestFile2, 'w') { |f| f.write yield }
  end

  def suppress_warning
    begin
      v = $VERBOSE
      $VERBOSE = false
      yield
    ensure
      $VERBOSE = v
    end
  end


  def setup
    @c = DummyContext.new
    @v = Wakaba::ERbView.new(TestFile)
  end


  def teardown
    File.unlink TestFile rescue nil
    File.unlink TestFile2 rescue nil
  end


  def test_plain
    create_test_file { 'Hello, World' }
    @v.run @c
    assert_equal 'Hello, World', @c.out
  end


  def test_print
    create_test_file { '<% print "Hello, World" %>' }
    @v.run @c
    assert_equal 'Hello, World', @c.out
  end


  def test_puts
    create_test_file { '<% puts "Hello, World" %>' }
    @v.run @c
    assert_equal "Hello, World\n", @c.out
  end


  def test_printf
    create_test_file { '<% printf "%s", "Hello, World" %>' }
    @v.run @c
    assert_equal 'Hello, World', @c.out
  end


  def test_aset
    value = 'fuga'
    assert_same value, (@v['hoge'] = value)
    create_test_file { '<%= hoge %>' }
    @v.run @c
    assert_equal 'fuga', @c.out
  end


  def test_raise_1
    create_test_file { '<% raise "hoge" %>' }
    e = assert_exception(RuntimeError) { @v.run @c }
    assert_equal 'hoge', e.message
    assert_equal "#{TestFile}:1:in `wakaba_binding'", e.backtrace[0]
    assert_equal "(eval):1:in `wakaba_binding'", e.backtrace[1]
  end

  def test_raise_2
    create_test_file {
      "<% def bar ; raise 'hoge' ; end %>\n" \
      "<% def foo ; bar ; end %>\n" \
      "<% foo %>"
    }
    e = assert_exception(RuntimeError) { @v.run @c }
    assert_equal 'hoge', e.message
    assert_equal "#{TestFile}:1:in `bar'", e.backtrace[0]
    if erb_version >= '1.4.0' then
      assert_equal "#{TestFile}:2:in `foo'", e.backtrace[1]
      assert_equal "#{TestFile}:3:in `wakaba_binding'", e.backtrace[2]
    else
      assert_equal "#{TestFile}:3:in `foo'", e.backtrace[1]
      assert_equal "#{TestFile}:5:in `wakaba_binding'", e.backtrace[2]
    end
    assert_equal "(eval):1:in `wakaba_binding'", e.backtrace[3]
  end


  def test_import_1
    create_test_file { "<% import '#{TestFile2}' %>" }
    create_test_file_2 { "<% print 'hoge' %>" }
    @v.run @c
    assert_equal 'hoge', @c.out
  end

  def test_import_2
    create_test_file { "<% import '#{TestFile2}' %>" }
    create_test_file_2 { "<% raise 'hoge' %>" }
    e = assert_exception(RuntimeError) { @v.run @c }
    assert_equal "#{TestFile2}:1:in `wakaba_binding'", e.backtrace[0]
    assert_equal "(eval):1:in `wakaba_binding'", e.backtrace[1]
  end

  def test_import_3
    create_test_file { "<% hoge = 'fuga' %><% import '#{TestFile2}' %>" }
    create_test_file_2 { "<% print hoge %>" }
    assert_no_exception { @v.run @c }
    assert_equal 'fuga', @c.out
  end

  def test_import_4
    create_test_file {
      "<% hoge = 'fuga' %>" \
      "<% def foo ; hoge = 'mogu' ; import '#{TestFile2}' ; end %>" \
      "<% foo %>"
    }
    create_test_file_2 { "<% print hoge %>" }
    assert_no_exception { @v.run @c }
    assert_equal 'fuga', @c.out
  end


  def test_pre_script_1
    class << @v
      public :pre_script
    end
    assert_nil @v.pre_script
  end

  def test_pre_script_2
    def @v.pre_script
      ['hoge = "mogu"']
    end
    create_test_file { '<% print hoge %>' }
    assert_no_exception { @v.run @c }
  end

  def test_post_script_1
    class << @v
      public :post_script
    end
    assert_nil @v.post_script
  end

  def test_post_script_2
    def @v.post_script
      ['print "hoge"']
    end
    create_test_file { '<% print "fuga" %>' }
    @v.run @c
    assert_equal "fugahoge", @c.out
  end

end




load "#{File.dirname($0)}/runtest.rb" if __FILE__ == $0
