#
# testpbinding.rb - test cases for Wakaba
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: testpbinding.rb,v 1.1 2002/07/29 10:49:55 katsu Exp $
#

require 'runit/testcase'
require 'wakaba/pbinding'


class TestProtectedBinding < RUNIT::TestCase

  def setup
    @b = Wakaba::ProtectedBinding.new
  end

  def test_eval
    assert_same true, @b.__eval__('true')
    assert_exception(NameError) { @b.__eval__ }
    assert_exception(NameError) { @b.wakaba_binding }
  end

  def test_eval_undefed
    assert_exception(NameError) { @b.__eval__('__eval__') }
  end

  def test_eval_undefed_2
    assert_exception(NameError) { @b.__eval__('wakaba_binding') }
  end

  def test_eval_assign
    begin
      eval 'hoge', TOPLEVEL_BINDING
      assert_fail "hoge is already defined at toplevel"
    rescue NameError
    end
    b1, b2 = @b.__eval__('hoge=1; [ TOPLEVEL_BINDING, binding ]')
    assert_equal 1, eval('hoge', b1)
    assert_equal 1, eval('hoge', b2)
    assert_exception(NameError) { eval('hoge', TOPLEVEL_BINDING) }
  end

  def test_eval_parameter
    fuga = 'fuga'
    bar = 'bar'
    v1, v2 = @b.__eval__('[ hoge, foo ]', { 'hoge' => fuga, 'foo' => bar })
    assert_same fuga, v1
    assert_same bar, v2
  end

  def test_eval_parameter_undefed
    assert_exception(NameError) { @b.__eval__('VARIABLES', {'hoge'=>'fuga'}) }
  end

  def test_eval_parameter_undefed_2
    assert_exception(NameError) {
      @b.__eval__('VARIABLES_', {'VARIABLES'=>'fuga'})
    }
  end

  def test_eval_define_method
    begin
      eval 'hoge', TOPLEVEL_BINDING
      assert_fail "hoge is already defined at toplevel"
    rescue NameError
    end
    v = @b.__eval__('def hoge; "fuga"; end; [hoge, TOPLEVEL_BINDING, binding]')
    assert_equal 'fuga', v[0]
    assert_equal 'fuga', eval('hoge', v[1])
    assert_equal 'fuga', eval('hoge', v[2])
    assert_exception(NameError) { eval('hoge', TOPLEVEL_BINDING) }
  end


  def test_raise
    e = assert_exception(RuntimeError) { @b.__eval__('raise "hoge"') }
    assert_equal "(eval):1:in `wakaba_binding': hoge", e.message
  end

  def test_raise_deep
    e = assert_exception(RuntimeError) {
      @b.__eval__("def bar ; raise 'hoge' ; end\n" \
                  "def foo ; bar ; end\n" \
                  "foo")
    }
    assert_equal "(eval):1:in `bar': hoge", e.message
  end

  def test_raise_file
    e = assert_exception(RuntimeError) {
      @b.__eval__('raise "hoge"', nil, 'foo')
    }
    assert_equal 'hoge', e.message
    assert_equal "foo:1:in `wakaba_binding'", e.backtrace[0]
    assert_equal "(eval):1:in `wakaba_binding'", e.backtrace[1]
  end

  def test_raise_file_2
    e = assert_exception(RuntimeError) {
      @b.__eval__("def bar ; raise 'hoge' ; end\n" \
                  "def foo ; bar ; end\n" \
                  "foo", nil, 'fuga')
    }
    assert_equal 'hoge', e.message
    assert_equal "fuga:1:in `bar'", e.backtrace[0]
    assert_equal "fuga:2:in `foo'", e.backtrace[1]
    assert_equal "fuga:3:in `wakaba_binding'", e.backtrace[2]
    assert_equal "(eval):1:in `wakaba_binding'", e.backtrace[3]
  end

  def test_raise_fileline
    e = assert_exception(RuntimeError) {
      @b.__eval__("def bar ; raise 'hoge' ; end\n" \
                  "def foo ; bar ; end\n" \
                  "foo", nil, 'fuga', 100)
    }
    assert_equal 'hoge', e.message
    assert_equal "fuga:100:in `bar'", e.backtrace[0]
    assert_equal "fuga:101:in `foo'", e.backtrace[1]
    assert_equal "fuga:102:in `wakaba_binding'", e.backtrace[2]
    assert_equal "(eval):1:in `wakaba_binding'", e.backtrace[3]
  end

end




class TestProtectedEval < RUNIT::TestCase

  def setup
    @e = Wakaba::ProtectedEval.new
  end


  def test_aset
    value = 'fuga'
    assert_same value, (@e['hoge'] = value)
    assert_same value, @e.eval('hoge')
  end

  def test_aset_fail
    assert_no_exception { @e['hoge'] = 'fuga' }
    assert_no_exception { @e[:hoge] = 'fuga' }
    assert_no_exception { @e['_hoge'] = 'fuga' }
    assert_exception(ArgumentError) { @e['Hoge'] = 'fuga' }
    assert_exception(ArgumentError) { @e['@hoge'] = 'fuga' }
    assert_exception(ArgumentError) { @e['0Hoge'] = 'fuga' }
  end

  def test_aset_overwrite
    @e['hoge'] = 'fuga'
    @e['hoge'] = 'moke'
    assert_equal 'moke', @e.eval('hoge')
  end

end




load "#{File.dirname($0)}/runtest.rb" if __FILE__ == $0
