#
# testpstore.rb - test cases for Wakaba
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: testpstore.rb,v 1.9 2002/09/10 13:16:12 katsu Exp $
#

require 'runit/testcase'
require 'wakaba/store/pstore'


class TestPStore < RUNIT::TestCase

  Record = Wakaba::Record
  WakabaVersion = Wakaba::Version.split('.')[0,2].join('.')

  TestFile = '/tmp/wakabapstoretest'
  Now = Time.now


  def setup
    @s0 = Wakaba::PStore.new(TestFile)
    @s1 = Wakaba::PStore.new(TestFile, 'hogeversion')
    @s2 = Wakaba::PStore.new(TestFile, 'hogeversion', 10)
    @s0.now = @s1.now = @s2.now = Now
    PStore.new(TestFile).transaction { |p|
      ver = ".#{WakabaVersion}"
      p['12345678'] = [ 'hogefuga', 'hogeskey', 'hogeversion'+ver, Now-10 ]
      p['23456789'] = [ 'hogefuga', 'hogeskey', 'fugaversion'+ver, Now-10 ]
      p['34567890'] = [ 'hogefuga', 'hogeskey', 'hogeversion'+ver, Now-20 ]
    }
  end


  def teardown
    File.unlink TestFile rescue nil
    File.unlink TestFile + '~' rescue nil
  end


  def test_open_ex
    @s0.open('12345678', true) { |record|
      assert_equal '12345678', record.index
      assert_equal 'hogefuga', record.object
      assert_equal 'hogeskey', record.server_key
      assert_equal "hogeversion.#{WakabaVersion}", record.version
      assert_equal Now-10, record.ctime
      assert_equal true, record.alive?
      record.object = 'hogefugahoge'
      record.server_key = 'hogefugaskey'
    }
    PStore.new(TestFile).transaction { |p|
      assert_equal([ 'hogefugahoge', 'hogefugaskey',
                     ".#{WakabaVersion}", Now ], p['12345678'])
    }
  end

  def test_open_ex_fail
    assert_exception(RuntimeError) {
      @s0.open('12345678', true) { |record|
        record.object = 'hogefugahoge'
        record.server_key = 'hogefugaskey'
        raise "dummy"
      }
    }
    PStore.new(TestFile).transaction { |p|
      assert_equal([ 'hogefuga', 'hogeskey',
                     "hogeversion.#{WakabaVersion}", Now-10 ], p['12345678'])
    }
  end

  def test_open_ex_alive
    @s0.open('12345678', true) { |record|
      record.object = 'hogefugahoge'
      record.server_key = 'hogefugaskey'
      record.alive = false
    }
    PStore.new(TestFile).transaction { |p|
      assert_equal false, p.root?('12345678')
    }
  end

  def test_open_ex_flock
    @s0.open('12345678', true) { |record|
      File.open(TestFile) { |f|
        assert_equal false, f.flock(File::LOCK_EX|File::LOCK_NB)
      }
    }
  end


  def test_open
    @s0.open('12345678', true) { |record|
      assert_equal '12345678', record.index
      assert_equal 'hogefuga', record.object
      assert_equal 'hogeskey', record.server_key
      assert_equal "hogeversion.#{WakabaVersion}", record.version
      assert_equal Now-10, record.ctime
      assert_equal true, record.alive?
      record.object = 'hogefugahoge'
      record.server_key = 'hogefugaskey'
    }
    PStore.new(TestFile).transaction { |p|
      assert_equal([ 'hogefugahoge', 'hogefugaskey',
                     ".#{WakabaVersion}", Now ], p['12345678'])
    }
  end

  def test_open_fail
    assert_exception(RuntimeError) {
      @s0.open('12345678', false) { |record|
        record.object = 'hogefugahoge'
        record.server_key = 'hogefugaskey'
        raise "dummy"
      }
    }
    PStore.new(TestFile).transaction { |p|
      assert_equal([ 'hogefuga', 'hogeskey',
                     "hogeversion.#{WakabaVersion}", Now-10 ], p['12345678'])
    }
  end

  def test_open_alive
    @s0.open('12345678', false) { |record|
      record.object = 'hogefugahoge'
      record.server_key = 'hogefugaskey'
      record.alive = false
    }
    PStore.new(TestFile).transaction { |p|
      assert_equal false, p.root?('12345678')
    }
  end

  def test_open_flock
    @s0.open('12345678', false) { |record|
      File.open(TestFile) { |f|
        assert_equal 0, f.flock(File::LOCK_EX|File::LOCK_NB)
      }
    }
  end


  def test_open_version
    assert_no_exception {
      @s1.open('12345678', true) { }
    }
    assert_exception(Wakaba::IncompatibleSession) {
      @s1.open('23456789', true) { }
    }
  end

  def test_open_maxage
    assert_no_exception {
      @s2.open('12345678', true) { }
    }
    assert_exception(Wakaba::ExpiredSession) {
      @s2.open('34567890', true) { }
    }
  end


  def test_create
    r = nil
    @s0.open(nil, nil) { |record|
      assert_match record.index, /\A[-_A-Za-z0-9]{8}\z/n
      assert_nil record.object
      assert_equal '', record.server_key
      assert_equal ".#{WakabaVersion}", record.version
      assert_equal Now, record.ctime
      assert_equal true, record.alive?
      record.object = 'hogefugahoge'
      record.server_key = 'hogefugaskey'
      r = record
    }
    PStore.new(TestFile).transaction { |p|
      assert_equal([ 'hogefugahoge', 'hogefugaskey',
                     ".#{WakabaVersion}", Now ], p[r.index])
    }
  end

  def test_create_fail
    r = nil
    assert_exception(RuntimeError) {
      @s0.open(nil, nil) { |record|
        r = record
        record.object = 'hogefugahoge'
        record.server_key = 'hogefugaskey'
        raise "dummy"
      }
    }
    PStore.new(TestFile).transaction { |p|
      assert_equal false, p.root?(r.index)
    }
  end

  def atest_open_alive
    r = nil
    @s0.open(nil, nil) { |record|
      r = record
      record.object = 'hogefugahoge'
      record.server_key = 'hogefugaskey'
      record.alive = false
    }
    PStore.new(TestFile).transaction { |p|
      assert_equal false, p.root?(r.index)
    }
  end

  def test_open_flock
    @s0.open(nil, nil) { |record|
      File.open(TestFile) { |f|
        assert_equal 0, f.flock(File::LOCK_EX|File::LOCK_NB)
      }
    }
  end

end




load "#{File.dirname($0)}/runtest.rb" if __FILE__ == $0
