#
# testsession.rb - test cases for Wakaba
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: testsession.rb,v 1.1 2002/09/10 11:07:28 katsu Exp $
#

require 'runit/testcase'
require 'asserttracecall'
require 'wakaba/session'


class TestSession < RUNIT::TestCase

  class Session < Wakaba::Session
    def welcome(c)
      'fuga'
    end
    def on_hoge(c)
      'hoge'
    end
  end

  class Context
    def initialize(action)
      @action = action
    end
    attr_reader :action
  end


  def setup
    @s = Session.new
  end

  def test_deny
    e = assert_exception(Wakaba::AccessDenied) { @s.instance_eval { deny } }
    assert_equal 'the access is not allowed', e.message
    e = assert_exception(Wakaba::AccessDenied) {
      @s.instance_eval { deny 'hogefuga' }
    }
    assert_equal 'hogefuga', e.message
  end

  def test_s_action_to_method
    assert_equal :welcome, Session.action_to_method('')
    assert_equal :on_hoge, Session.action_to_method('hoge')
    assert_equal 'on_hoge.html'.intern, Session.action_to_method('hoge.html')
  end

  def test_s_method_to_action
    assert_equal 'hoge', Session.method_to_action('hoge')
    assert_equal 'hoge', Session.method_to_action(:on_hoge)
    assert_equal '', Session.method_to_action(:welcome)
    assert_exception(Wakaba::ActionNameError) {
      Session.method_to_action(:hoge)
    }
    assert_exception(TypeError) { Session.method_to_action(1) }
  end

  def test_dispatch
    c = Context.new('hoge')
    assert_equal 'hoge', @s.dispatch(c)
    assert_trace_call([@s, :setup, {'context'=>c}],
                      [@s, :on_hoge, {'c'=>c}],
                      [@s, :teardown, {'context'=>c}]) { @s.dispatch c }
  end

  def test_dispatch_welcome
    c = Context.new('')
    assert_equal 'fuga', @s.dispatch(c)
    assert_trace_call([@s, :setup, {'context'=>c}],
                      [@s, :welcome, {'c'=>c}],
                      [@s, :teardown, {'context'=>c}]) { @s.dispatch c }
  end

  def test_dispatch_fail
    c = Context.new('fuga')
    assert_exception(Wakaba::ActionNotFound) { @s.dispatch c }
    assert_trace_call([@s, :action_missing, {'context'=>c}]) {
      @s.dispatch c rescue nil
    }
  end

  def test_dispatch_raise
    c = Context.new('hoge')
    def @s.on_hoge(c)
      raise 'dummy'
    end
    assert_exception(RuntimeError) { @s.dispatch c }
    assert_trace_call([@s, :setup, {'context'=>c}],
                      [@s, :on_hoge, {'c'=>c}],
                      [@s, :teardown, {'context'=>c}]) {
      @s.dispatch c rescue nil
    }
  end

  def test_dispatch_raise_setup
    c = Context.new('hoge')
    def @s.setup(c)
      raise 'dummy'
    end
    assert_exception(RuntimeError) { @s.dispatch c }
    assert_trace_call([@s, :setup, {'c'=>c}]) {
      @s.dispatch c rescue nil
    }
  end

end




class TestSessionMap < RUNIT::TestCase

  def setup
    @s1 = Wakaba::Session.new
    @s2 = Wakaba::Session.new
    @s3 = Wakaba::Session.new
    @hash = { '' => @s1, '/hoge' => @s2, '/fuga' => @s3 }
    @map = Wakaba::SessionMap.new(@hash.dup)
  end

  def test_s_new
    assert_equal @hash, @map.hash
  end

  def test_s_new_omit
    assert_equal({}, Wakaba::SessionMap.new.hash)
  end

  def test_hash
    assert_equal @hash, @map.hash
  end

  def test_lookup
    assert_equal @s1, @map.lookup('')
    assert_equal @s2, @map.lookup('/hoge')
    assert_equal @s3, @map.lookup('/fuga')
    assert_exception(Wakaba::SessionNameError) { @map.lookup '/foo' }
  end

  def test_mount
    s4 = Wakaba::Session.new
    s5 = Wakaba::Session.new
    assert_equal '/foo', @map.mount(s4, '/foo')
    assert_exception(Wakaba::SessionTypeError) { @map.mount s4, '/bar' }
    assert_exception(Wakaba::SessionNameError) { @map.mount s5, '/foo' }
    assert_equal '/foo', @map.mount(s4, '/foo')
    assert_exception(Wakaba::SessionNameError) { @map.mount s5, '' }
    assert_exception(Wakaba::SessionNameError) { @map.mount s5, '/' }
    assert_exception(Wakaba::SessionNameError) { @map.mount s5, '/hoge/fuga/' }
    assert_exception(Wakaba::SessionNameError) { @map.mount s5, 'hoge' }
    assert_exception(Wakaba::SessionNameError) { @map.mount s5, '/ho ge' }
    assert_no_exception { @map.mount s5, '/ho.ge' }
    assert_equal '', @map.mount(@s1)
    assert_equal '/hoge', @map.mount(@s2)
    assert_equal '/fuga', @map.mount(@s3)
    assert_equal '/foo', @map.mount(s4)
    assert_equal '/ho.ge', @map.mount(s5)
    h = @hash.dup.update({'/foo'=>s4,'/ho.ge'=>s5})
    assert_equal h, @map.hash
  end

  def test_mount_omit
    s4 = Wakaba::Session.new
    s5 = Wakaba::Session.new
    s6 = Wakaba::Session.new
    s7 = Wakaba::Session.new
    assert_equal '/Session', @map.mount(s4)
    assert_equal '/Session-0001', @map.mount(s5)
    assert_equal '/Session-0002', @map.mount(s6, '/Session-0002')
    assert_equal '/Session-0003', @map.mount(s7)
    assert_equal '/Session', @map.mount(s4)
    h = @hash.dup.update({'/Session'=>s4,'/Session-0001'=>s5,
                          '/Session-0002'=>s6,'/Session-0003'=>s7})
    assert_equal h, @map.hash
  end

  def test_umount
    s4 = Wakaba::Session.new
    assert_equal '/hoge', @map.umount(@s2)
    assert_equal @hash.reject{|k,v|k=='/hoge'}, @map.hash
    assert_exception(Wakaba::SessionTypeError) { @map.umount s4 }
  end

end




load "#{File.dirname($0)}/runtest.rb" if __FILE__ == $0
