#
# teststore.rb - test cases for Wakaba
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: teststore.rb,v 1.2 2002/09/10 13:16:12 katsu Exp $
#

require 'runit/testcase'
require 'asserttracecall'
require 'wakaba/store'


class TestRecord < RUNIT::TestCase

  def setup
    @o = Object.new
    @ctime = Time.now
    @r = Wakaba::Record.new('hoge', @o, 'fuga', 'foo', @ctime)
  end

  def test_initialize
    assert_equal 'hoge', @r.index
    assert_same @o, @r.object
    assert_equal 'fuga', @r.server_key
    assert_equal 'foo', @r.version
    assert_equal @ctime, @r.ctime
    assert_equal true, @r.alive?
  end

  def test_s_aref
    r = Wakaba::Record['hoge', @o, 'fuga', 'foo', @ctime]
    assert_kind_of Wakaba::Record, r
    assert_equal 'hoge', @r.index
    assert_same @o, @r.object
    assert_equal 'fuga', @r.server_key
    assert_equal 'foo', @r.version
    assert_equal @ctime, @r.ctime
    assert_equal true, @r.alive?
  end

  def test_initialize_omit
    r = Wakaba::Record.new('hoge', @o)
    assert_equal '', r.server_key
    assert_equal '', r.version
    assert_equal Time.at(0), r.ctime
    assert_equal true, @r.alive?
  end

  def test_index_eq
    v = 'peke'
    assert_equal v, @r.index = v
    assert_equal 'peke', @r.index
    assert_exception(TypeError) { @r.index = nil }
    assert_exception(TypeError) { @r.index = 1 }
  end

  def test_server_key_eq
    v = 'peke'
    assert_same v, @r.server_key = v
    assert_equal 'peke', @r.server_key
    assert_exception(TypeError) { @r.server_key = nil }
    assert_exception(TypeError) { @r.server_key = 1 }
  end

  def test_version_eq
    v = 'peke'
    assert_same v, @r.version = v
    assert_equal 'peke', @r.version
    assert_exception(TypeError) { @r.version = nil }
    assert_exception(TypeError) { @r.version = 1 }
  end

  def test_ctime_eq
    t = @ctime - 10
    assert_same t, @r.ctime = t
    assert_equal t, @r.ctime
    assert_exception(TypeError) { @r.index = nil }
    assert_exception(TypeError) { @r.index = 1 }
  end

  def test_object_eq
    o = Object.new
    assert_same o, @r.object = o
    assert_same o, @r.object
    assert_no_exception { @r.object = nil }
    assert_same nil, @r.object
  end

  def test_identity
    assert_equal '', @r.identity
  end

  def test_alive_eq
    o = Object.new
    assert_equal true, @r.alive
    assert_equal true, @r.alive?
    assert_same o, @r.alive = o
    assert_equal true, @r.alive?
    assert_same false, @r.alive = false
    assert_equal false, @r.alive?
    assert_same nil, @r.alive = nil
    assert_equal false, @r.alive?
  end

end




class TestMarshallingRecord < RUNIT::TestCase

  def setup
    @o = Object.new
    @m = Marshal.dump(@o)
    @ctime = Time.now
    @r = Wakaba::MarshallingRecord.new('hoge', @m, 'fuga', 'foo', @ctime)
  end

  def test_initialize_nil
    @r = Wakaba::MarshallingRecord.new('hoge')
    assert_nil @r.object
  end

  def test_marshal_object
    assert_same @m, @r.marshal_object
  end

  def test_marshal_object_read
    @r.object
    assert_equal @m, @r.marshal_object
    assert_exception(RUNIT::AssertionFailedError) {
      assert_same @m, @r.marshal_object
    }
  end

  def test_marshal_object_write
    @r.object = @o
    assert_equal @m, @r.marshal_object
    assert_exception(RUNIT::AssertionFailedError) {
      assert_same @m, @r.marshal_object
    }
  end

  def test_identity
    assert_equal @m, @r.identity
  end

end




class TestStore < RUNIT::TestCase

  WakabaVersion = Wakaba::Version.split('.')[0,2].join('.')

  def setup
    @s = Wakaba::Store.new
  end

  def test_now_eq
    t = Time.at(12345678)
    assert_same t, @s.now = t
    assert_equal t, @s.now
  end

  def test_now_eq_fail
    assert_exception(TypeError) { @s.now = 1 }
  end

  def test_now_1
    assert_kind_of Time, @s.now
    assert_trace_call([Time, :now]) { @s.now }
  end

  def test_now_2
    t = Time.at(12345678)
    @s.now = t
    assert_equal t, @s.now
    assert_exception(RUNIT::AssertionFailedError) {
      assert_trace_call([Time, :now]) { @s.now }
    }
    @s.now = nil
    assert_kind_of Time, @s.now
    assert_trace_call([Time, :now]) { @s.now }
  end

  def test_version
    assert_equal ".#{WakabaVersion}", @s.version
    assert_equal "hoge.#{WakabaVersion}", Wakaba::Store.new('hoge').version
  end


  def test_check_record
    assert_no_exception {
      @s.send :check_record, Wakaba::Record.new('foo',nil)
    }
    assert_no_exception {
      @s.send :check_record, Wakaba::Record.new('foo',nil,nil,'a',Time.now)
    }
  end

  def test_check_record_version
    @s = Wakaba::Store.new('bar')
    assert_exception(Wakaba::IncompatibleSession) {
      @s.send :check_record, Wakaba::Record.new('foo',nil)
    }
    assert_exception(Wakaba::IncompatibleSession) {
      @s.send :check_record, Wakaba::Record.new('foo',nil,nil,'bar')
    }
    assert_no_exception {
      ver = "bar.#{WakabaVersion}"
      @s.send :check_record, Wakaba::Record.new('foo',nil,nil,ver)
    }
  end

  def test_check_record_age
    @s = Wakaba::Store.new(nil, 100)
    @s.now = Time.at(1000)
    assert_exception(Wakaba::ExpiredSession) {
      @s.send :check_record, Wakaba::Record.new('foo',nil)
    }
    assert_exception(Wakaba::ExpiredSession) {
      @s.send :check_record, Wakaba::Record.new('foo',nil,nil,nil,Time.at(889))
    }
    assert_no_exception {
      @s.send :check_record, Wakaba::Record.new('foo',nil,nil,nil,Time.at(900))
    }
  end


  def test_lock_failed
    assert_exception(Wakaba::LockFailed) {
      @s.send :lock_failed
    }
  end


  class Store < Wakaba::Store
    def _record=(r)
      @_record = r
    end
    def save
    end
    private :save
    def do_open(index)
      raise "error" unless index == @_record.index
      yield @_record
      save
    end
    def do_open_ex(index)
      raise "error" unless index == @_record.index
      yield @_record
      save
    end
    def do_create
      yield @_record
      save
    end
  end

  def dummy
    # dummy method for tracing method call.
  end
  def dummy2
    # dummy method for tracing method call.
  end


  def test_open
    @s = Store.new
    @s._record = r = Wakaba::Record.new('hoge', Object.new)
    assert_trace_call([ @s, :do_open_ex, {'index'=>'hoge'} ],
                      [ @s, :check_record, {'record'=>r} ],
                      [ self, :dummy ],
                      [ @s, :save ],
                      [ self, :dummy2 ]) {
      @s.open('hoge', true) { dummy }
      dummy2
    }
  end

  def test_open_unlock
    @s = Store.new
    @s._record = r = Wakaba::Record.new('hoge', Object.new)
    assert_trace_call([ @s, :do_open, {'index'=>'hoge'} ],
                      [ @s, :check_record, {'record'=>r} ],
                      [ self, :dummy ],
                      [ @s, :save ],
                      [ self, :dummy2 ]) {
      @s.open('hoge', false) { dummy }
      dummy2
    }
  end

  def test_open_fail_checkrecord
    @s = Store.new('foo')
    @s._record = r = Wakaba::Record.new('hoge', Object.new, nil, 'bar')
    assert_trace_call([ @s, :do_open_ex, {'index'=>'hoge'} ],
                      [ @s, :check_record, {'record'=>r} ],
                      [ @s, :save ],
                      [ self, :dummy2 ]) {
      assert_exception(Wakaba::IncompatibleSession) {
        @s.open('hoge', true) { dummy }
      }
      dummy2
    }
  end

  def test_open_fail_yield
    @s = Store.new
    @s._record = r = Wakaba::Record.new('hoge', Object.new)
    assert_trace_call([ @s, :do_open_ex, {'index'=>'hoge'} ],
                      [ @s, :check_record, {'record'=>r} ],
                      [ self, :dummy ],
                      [ self, :dummy2 ]) {
      assert_exception(RuntimeError) {
        @s.open('hoge', true) { dummy ; raise "dummy" }
      }
      dummy2
    }
  end

  def test_open_record
    @s = Store.new('foo')
    @s.now = Time.at(12345678)
    @s._record = r = Wakaba::Record.new('hoge', Object.new, nil, @s.version)
    orig_index = r.index
    @s.open('hoge', true) { |record|
      assert_same r, record
      assert_same orig_index, record.index
    }
    assert_exception(RUNIT::AssertionFailedError) {
      assert_same orig_index, r.index
    }
    assert_equal Time.at(12345678), r.ctime
    assert_equal "foo.#{WakabaVersion}", r.version
  end


  def test_create
    @s = Store.new
    @s._record = Wakaba::Record.new('hoge', Object.new)
    assert_trace_call([ @s, :do_create ],
                      [ self, :dummy ],
                      [ @s, :save ],
                      [ self, :dummy2 ]) {
      @s.open(nil, nil) { dummy }
      dummy2
    }
  end

  def test_create_fail_yield
    @s = Store.new
    @s._record = Wakaba::Record.new('hoge', Object.new)
    assert_trace_call([ @s, :do_create ],
                      [ self, :dummy ],
                      [ self, :dummy2 ]) {
      assert_exception(RuntimeError) {
        @s.open(nil, nil) { dummy ; raise "dummy" }
      }
      dummy2
    }
  end

  def test_create_record
    @s = Store.new('foo')
    @s.now = Time.at(12345678)
    @s._record = r = Wakaba::Record.new('hoge', Object.new)
    orig_index = nil
    @s.open(nil, nil) { |record|
      assert_same r, record
      orig_index = record.index
    }
    assert_exception(RUNIT::AssertionFailedError) {
      assert_same orig_index, r.index
    }
    assert_equal Time.at(12345678), r.ctime
    assert_equal "foo.#{WakabaVersion}", r.version
  end


  def test_do_open
    assert_exception(Wakaba::LockUnsupported) { @s.send :do_open, 'hoge' }
  end

  def test_do_open_ex
    assert_exception(Wakaba::LockUnsupported) { @s.send :do_open_ex, 'hoge' }
  end

  def test_do_create
    assert_exception(Wakaba::AbstractMethod) { @s.send :do_create }
  end

end




class TestEchoStore < RUNIT::TestCase

  WakabaVersion = Wakaba::Version.split('.')[0,2].join('.')

  def setup
    @s = Wakaba::EchoStore.new
    @r = @s.open(nil, nil) { |record|
      record.object = ['hoge','fuga']
      record.server_key = 'peke'
      record
    }
  end

  def test_create
    s = [Marshal.dump(['hoge','fuga'])].pack('m*').gsub(/\s+/,'').tr('=','_')
    assert_equal "#{@r.ctime.to_i}-#{s}-peke-.#{WakabaVersion}", @r.index
  end

  def test_create_escape
    r = @s.open(nil, nil) { |record|
      record.object = ['hoge','fuga']
      record.server_key = 'peke-po.n'
      record
    }
    s = [Marshal.dump(['hoge','fuga'])].pack('m*').gsub(/\s+/,'').tr('=','_')
    assert_equal "#{r.ctime.to_i}-#{s}-peke._po..n-.#{WakabaVersion}",r.index
  end

  def test_open
    @s.open(@r.index, false) { |r|
      assert_equal @r.index, r.index
      assert_equal 'peke', r.server_key
      assert_equal ".#{WakabaVersion}", r.version
      assert_equal @r.ctime, r.ctime
    }
  end

  def test_open_fail
    assert_exception(Wakaba::InvalidSessionIndex) { @s.open 'hoge', false }
  end

  def test_open_lock
    assert_exception(Wakaba::LockUnsupported) { @s.open @r.index, true }
  end

end




load "#{File.dirname($0)}/runtest.rb" if __FILE__ == $0
